/*
 * Copyright (C) 2015 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.android.setupwizardlib;

import android.content.Context;
import android.content.res.TypedArray;
import android.util.AttributeSet;

import com.android.setupwizardlib.items.ItemAdapter;
import com.android.setupwizardlib.items.ItemGroup;
import com.android.setupwizardlib.items.ItemInflater;

public class SetupWizardItemsLayout extends SetupWizardListLayout {

    private ItemAdapter mAdapter;

    public SetupWizardItemsLayout(Context context, AttributeSet attrs) {
        super(context, attrs);
        init(context, attrs, 0);
    }

    public SetupWizardItemsLayout(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init(context, attrs, defStyleAttr);
    }

    private void init(Context context, AttributeSet attrs, int defStyleAttr) {
        TypedArray a = context.obtainStyledAttributes(attrs, R.styleable.SuwSetupWizardItemsLayout,
                defStyleAttr, 0);
        int xml = a.getResourceId(R.styleable.SuwSetupWizardItemsLayout_android_entries, 0);
        if (xml != 0) {
            ItemGroup inflated = (ItemGroup) new ItemInflater(context).inflate(xml);
            mAdapter = new ItemAdapter(inflated);
            setAdapter(mAdapter);
        }
        a.recycle();
    }

    public ItemAdapter getAdapter() {
        return mAdapter;
    }
}
